using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Comrade.nrControls
{
	public enum Shape
	{
		Rectangle,
		Ellipse,
		Diamond,
		Triangle,
		Pentagon,
		Arrow
	}

	public enum Direction
	{
		Up,
		Down,
		Left,
		Right,
		UpperLeft,
		UpperRight,
		LowerLeft,
		LowerRight
	}

	/// <summary>
	/// Class1 ̊Tv̐łB
	/// </summary>
	public class nrDrawing
	{
		public nrDrawing()
		{
			// 
			// TODO: RXgN^ WbNɒǉĂB
			//
		}

		public static SolidBrush lightestBrush = new SolidBrush(Color.FromArgb(127, SystemColors.ControlLightLight));
		public static SolidBrush lighterBrush = new SolidBrush(Color.FromArgb(63, SystemColors.ControlLight));
		public static SolidBrush darkerBrush = new SolidBrush(Color.FromArgb(63, SystemColors.ControlDark));
		public static SolidBrush darkestBrush = new SolidBrush(Color.FromArgb(127, SystemColors.ControlDarkDark));
	
		public static Pen thickLightestPen = new Pen(lightestBrush, 3);
		public static Pen thickLighterPen = new Pen(lighterBrush, 3);
		public static Pen thickDarkerPen = new Pen(darkerBrush, 3);
		public static Pen thickDarkestPen = new Pen(darkestBrush, 3);

		public static Pen thinLightestPen = new Pen(lightestBrush, 1);
		public static Pen thinLighterPen = new Pen(lighterBrush, 1);
		public static Pen thinDarkerPen = new Pen(darkerBrush, 1);
		public static Pen thinDarkestPen = new Pen(darkestBrush, 1);

		public int rectCenter = 0;
		public int rectMiddle = 0;
		public int rectMin = 0;

		public Point[] diamondPoints = new Point[4];
		public Point[] trianglePoints = new Point[3];
		public Point[] pentagonPoints = new Point[5];
		public Point[] arrowPoints = new Point[7];

		public Rectangle ClientRectangle = new Rectangle(0, 0, 0, 0);

		public Shape shape = Shape.Rectangle;
		public Direction direction = Direction.Up;

		public void SetPath(GraphicsPath path)
		{
			if (shape == Shape.Ellipse)
				path.AddEllipse(ClientRectangle);
			else
			{
				rectCenter = ClientRectangle.Left + ClientRectangle.Width / 2;
				rectMiddle = ClientRectangle.Top + ClientRectangle.Height / 2;
				rectMin = Math.Min(ClientRectangle.Height, ClientRectangle.Width) / 2;
				switch (shape)
				{
				case Shape.Diamond:
					SetDiamondPoints();
					path.AddPolygon(diamondPoints);
					break;
				case Shape.Triangle:
					SetTrianglePoints();
					path.AddPolygon(trianglePoints);
					break;
				case Shape.Pentagon:
					SetPentagonPoints();
					path.AddPolygon(pentagonPoints);
					break;
				case Shape.Arrow:
					SetArrowPoints();
					path.AddPolygon(arrowPoints);
					break;
				}
			}
		}

		public void SetDiamondPoints()
		{
			diamondPoints[0] = new Point(rectCenter, ClientRectangle.Top);
			diamondPoints[1] = new Point(ClientRectangle.Left, rectMiddle);
			diamondPoints[2] = new Point(rectCenter, ClientRectangle.Bottom);
			diamondPoints[3] = new Point(ClientRectangle.Right, rectMiddle);
		}

		public void SetTrianglePoints()
		{
			switch (direction)
			{
			case Direction.Up:
				trianglePoints[0] = new Point(rectCenter, ClientRectangle.Top);
				trianglePoints[1] = new Point(ClientRectangle.Left, ClientRectangle.Bottom);
				trianglePoints[2] = new Point(ClientRectangle.Right, ClientRectangle.Bottom);
				break;
			case Direction.Down:
				trianglePoints[0] = new Point(rectCenter, ClientRectangle.Bottom);
				trianglePoints[1] = new Point(ClientRectangle.Right, ClientRectangle.Top);
				trianglePoints[2] = new Point(ClientRectangle.Left, ClientRectangle.Top);
				break;
			case Direction.Left:
				trianglePoints[0] = new Point(ClientRectangle.Left, rectMiddle);
				trianglePoints[1] = new Point(ClientRectangle.Right, ClientRectangle.Bottom);
				trianglePoints[2] = new Point(ClientRectangle.Right, ClientRectangle.Top);
				break;
			case Direction.Right:
				trianglePoints[0] = new Point(ClientRectangle.Right, rectMiddle);
				trianglePoints[1] = new Point(ClientRectangle.Left, ClientRectangle.Top);
				trianglePoints[2] = new Point(ClientRectangle.Left, ClientRectangle.Bottom);
				break;
			}
		}

		public void SetPentagonPoints()
		{
			int i;
			switch (direction)
			{
			case Direction.Up:
				i = ClientRectangle.Top + rectMin;
				pentagonPoints[0] = new Point(rectCenter, ClientRectangle.Top);
				pentagonPoints[1] = new Point(ClientRectangle.Left, i);
				pentagonPoints[2] = new Point(ClientRectangle.Left, ClientRectangle.Bottom);
				pentagonPoints[3] = new Point(ClientRectangle.Right, ClientRectangle.Bottom);
				pentagonPoints[4] = new Point(ClientRectangle.Right, i);
				break;
			case Direction.Down:
				i = ClientRectangle.Bottom - rectMin;
				pentagonPoints[0] = new Point(rectCenter, ClientRectangle.Bottom);
				pentagonPoints[1] = new Point(ClientRectangle.Right, i);
				pentagonPoints[2] = new Point(ClientRectangle.Right, ClientRectangle.Top);
				pentagonPoints[3] = new Point(ClientRectangle.Left, ClientRectangle.Top);
				pentagonPoints[4] = new Point(ClientRectangle.Left, i);
				break;
			case Direction.Left:
				i = ClientRectangle.Left + rectMin;
				pentagonPoints[0] = new Point(ClientRectangle.Left, rectMiddle);
				pentagonPoints[1] = new Point(i, ClientRectangle.Bottom);
				pentagonPoints[2] = new Point(ClientRectangle.Right, ClientRectangle.Bottom);
				pentagonPoints[3] = new Point(ClientRectangle.Right, ClientRectangle.Top);
				pentagonPoints[4] = new Point(i, ClientRectangle.Top);
				break;
			case Direction.Right:
				i = ClientRectangle.Right - rectMin;
				pentagonPoints[0] = new Point(ClientRectangle.Right, rectMiddle);
				pentagonPoints[1] = new Point(i, ClientRectangle.Top);
				pentagonPoints[2] = new Point(ClientRectangle.Left, ClientRectangle.Top);
				pentagonPoints[3] = new Point(ClientRectangle.Left, ClientRectangle.Bottom);
				pentagonPoints[4] = new Point(i, ClientRectangle.Bottom);
				break;
			}
		}

		public void SetArrowPoints()
		{
			int i, j, k;
			switch (direction)
			{
			case Direction.Up:
				i = ClientRectangle.Top + rectMin;
				j = ClientRectangle.Left + ClientRectangle.Width / 4;
				k = ClientRectangle.Right - ClientRectangle.Width / 4;
				arrowPoints[0] = new Point(rectCenter, ClientRectangle.Top);
				arrowPoints[1] = new Point(ClientRectangle.Left, i);
				arrowPoints[2] = new Point(j, i);
				arrowPoints[3] = new Point(j, ClientRectangle.Bottom);
				arrowPoints[4] = new Point(k, ClientRectangle.Bottom);
				arrowPoints[5] = new Point(k, i);
				arrowPoints[6] = new Point(ClientRectangle.Right, i);
				break;
			case Direction.Down:
				i = ClientRectangle.Bottom - rectMin;
				j = ClientRectangle.Right - ClientRectangle.Width / 4;
				k = ClientRectangle.Left + ClientRectangle.Width / 4;
				arrowPoints[0] = new Point(rectCenter, ClientRectangle.Bottom);
				arrowPoints[1] = new Point(ClientRectangle.Right, i);
				arrowPoints[2] = new Point(j, i);
				arrowPoints[3] = new Point(j, ClientRectangle.Top);
				arrowPoints[4] = new Point(k, ClientRectangle.Top);
				arrowPoints[5] = new Point(k, i);
				arrowPoints[6] = new Point(ClientRectangle.Left, i);
				break;
			case Direction.Left:
				i = ClientRectangle.Left + rectMin;
				j = ClientRectangle.Bottom - ClientRectangle.Height / 4;
				k = ClientRectangle.Top + ClientRectangle.Height / 4;
				arrowPoints[0] = new Point(ClientRectangle.Left, rectMiddle);
				arrowPoints[1] = new Point(i, ClientRectangle.Bottom);
				arrowPoints[2] = new Point(i, j);
				arrowPoints[3] = new Point(ClientRectangle.Right, j);
				arrowPoints[4] = new Point(ClientRectangle.Right, k);
				arrowPoints[5] = new Point(i, k);
				arrowPoints[6] = new Point(i, ClientRectangle.Top);
				break;
			case Direction.Right:
				i = ClientRectangle.Right - rectMin;
				j = ClientRectangle.Top + ClientRectangle.Height / 4;
				k = ClientRectangle.Bottom - ClientRectangle.Height / 4;
				arrowPoints[0] = new Point(ClientRectangle.Right, rectMiddle);
				arrowPoints[1] = new Point(i, ClientRectangle.Top);
				arrowPoints[2] = new Point(i, j);
				arrowPoints[3] = new Point(ClientRectangle.Left, j);
				arrowPoints[4] = new Point(ClientRectangle.Left, k);
				arrowPoints[5] = new Point(i, k);
				arrowPoints[6] = new Point(i, ClientRectangle.Bottom);
				break;
			}
		}

		public void Draw3DEllipse(Graphics graphics, bool convex)
		{
			Pen pen1 = convex ? thinLightestPen : thinDarkestPen;
			Pen pen2 = convex ? thinLighterPen : thinDarkerPen;
			Pen pen3 = convex ? thinDarkerPen : thinLighterPen;
			Pen pen4 = convex ? thinDarkestPen : thinLightestPen;

			Rectangle innerRect = Rectangle.Inflate(ClientRectangle, 0, 0);
			graphics.DrawArc(pen1, innerRect,  180,  90);
			graphics.DrawArc(pen4, innerRect,    0,  90);

			innerRect.Inflate(-1, -1);
			graphics.DrawArc(pen1, innerRect,  165, 120);
			graphics.DrawArc(pen4, innerRect,  -15, 120);

			innerRect.Inflate(-1, -1);
			graphics.DrawArc(pen2, innerRect,  150, 150);
			graphics.DrawArc(pen3, innerRect,  -30, 150);
		}

		public void Draw3DPolygon(Graphics graphics, bool convex)
		{
			Pen[] pen =
			{
				convex ? thickLightestPen : thickDarkestPen,
				convex ? thickLighterPen : thickDarkerPen,
				convex ? thickDarkerPen : thickLighterPen,
				convex ? thickDarkestPen : thickLightestPen
			};

			switch (shape)
			{
			case Shape.Diamond:
				Draw3DDiamond(graphics, pen);
				break;
			case Shape.Triangle:
				Draw3DTriangle(graphics, pen);
				break;
			case Shape.Pentagon:
				Draw3DPentagon(graphics, pen);
				break;
			case Shape.Arrow:
				Draw3DArrow(graphics, pen);
				break;
			}
		}

		private void Draw3DDiamond(Graphics graphics, Pen[] pen)
		{
			graphics.DrawLine(pen[0], diamondPoints[0], diamondPoints[1]);
			graphics.DrawLine(pen[2], diamondPoints[1], diamondPoints[2]);
			graphics.DrawLine(pen[3], diamondPoints[2], diamondPoints[3]);
			graphics.DrawLine(pen[1], diamondPoints[3], diamondPoints[0]);
		}

		private void Draw3DTriangle(Graphics graphics, Pen[] pen)
		{
			switch (direction)
			{
			case Direction.Up:
				graphics.DrawLine(pen[0], trianglePoints[0], trianglePoints[1]);
				graphics.DrawLine(pen[3], trianglePoints[1], trianglePoints[2]);
				graphics.DrawLine(pen[1], trianglePoints[2], trianglePoints[0]);
				break;
			case Direction.Down:
				graphics.DrawLine(pen[3], trianglePoints[0], trianglePoints[1]);
				graphics.DrawLine(pen[0], trianglePoints[1], trianglePoints[2]);
				graphics.DrawLine(pen[2], trianglePoints[2], trianglePoints[0]);
				break;
			case Direction.Left:
				graphics.DrawLine(pen[2], trianglePoints[0], trianglePoints[1]);
				graphics.DrawLine(pen[3], trianglePoints[1], trianglePoints[2]);
				graphics.DrawLine(pen[0], trianglePoints[2], trianglePoints[0]);
				break;
			case Direction.Right:
				graphics.DrawLine(pen[1], trianglePoints[0], trianglePoints[1]);
				graphics.DrawLine(pen[0], trianglePoints[1], trianglePoints[2]);
				graphics.DrawLine(pen[3], trianglePoints[2], trianglePoints[0]);
				break;
			}
		}

		private void Draw3DPentagon(Graphics graphics, Pen[] pen)
		{
			switch (direction)
			{
			case Direction.Up:
				graphics.DrawLine(pen[0], pentagonPoints[0], pentagonPoints[1]);
				graphics.DrawLine(pen[0], pentagonPoints[1], pentagonPoints[2]);
				graphics.DrawLine(pen[3], pentagonPoints[2], pentagonPoints[3]);
				graphics.DrawLine(pen[3], pentagonPoints[3], pentagonPoints[4]);
				graphics.DrawLine(pen[1], pentagonPoints[4], pentagonPoints[0]);
				break;
			case Direction.Down:
				graphics.DrawLine(pen[3], pentagonPoints[0], pentagonPoints[1]);
				graphics.DrawLine(pen[3], pentagonPoints[1], pentagonPoints[2]);
				graphics.DrawLine(pen[0], pentagonPoints[2], pentagonPoints[3]);
				graphics.DrawLine(pen[0], pentagonPoints[3], pentagonPoints[4]);
				graphics.DrawLine(pen[2], pentagonPoints[4], pentagonPoints[0]);
				break;
			case Direction.Left:
				graphics.DrawLine(pen[2], pentagonPoints[0], pentagonPoints[1]);
				graphics.DrawLine(pen[3], pentagonPoints[1], pentagonPoints[2]);
				graphics.DrawLine(pen[3], pentagonPoints[2], pentagonPoints[3]);
				graphics.DrawLine(pen[0], pentagonPoints[3], pentagonPoints[4]);
				graphics.DrawLine(pen[0], pentagonPoints[4], pentagonPoints[0]);
				break;
			case Direction.Right:
				graphics.DrawLine(pen[1], pentagonPoints[0], pentagonPoints[1]);
				graphics.DrawLine(pen[0], pentagonPoints[1], pentagonPoints[2]);
				graphics.DrawLine(pen[0], pentagonPoints[2], pentagonPoints[3]);
				graphics.DrawLine(pen[3], pentagonPoints[3], pentagonPoints[4]);
				graphics.DrawLine(pen[3], pentagonPoints[4], pentagonPoints[0]);
				break;
			}
		}

		private void Draw3DArrow(Graphics graphics, Pen[] pen)
		{
			switch (direction)
			{
			case Direction.Up:
				graphics.DrawLine(pen[0], arrowPoints[0], arrowPoints[1]);
				graphics.DrawLine(pen[3], arrowPoints[1], arrowPoints[2]);
				graphics.DrawLine(pen[0], arrowPoints[2], arrowPoints[3]);
				graphics.DrawLine(pen[3], arrowPoints[3], arrowPoints[4]);
				graphics.DrawLine(pen[3], arrowPoints[4], arrowPoints[5]);
				graphics.DrawLine(pen[3], arrowPoints[5], arrowPoints[6]);
				graphics.DrawLine(pen[1], arrowPoints[6], arrowPoints[0]);
				break;
			case Direction.Down:
				graphics.DrawLine(pen[3], arrowPoints[0], arrowPoints[1]);
				graphics.DrawLine(pen[0], arrowPoints[1], arrowPoints[2]);
				graphics.DrawLine(pen[3], arrowPoints[2], arrowPoints[3]);
				graphics.DrawLine(pen[0], arrowPoints[3], arrowPoints[4]);
				graphics.DrawLine(pen[0], arrowPoints[4], arrowPoints[5]);
				graphics.DrawLine(pen[0], arrowPoints[5], arrowPoints[6]);
				graphics.DrawLine(pen[2], arrowPoints[6], arrowPoints[0]);
				break;
			case Direction.Left:
				graphics.DrawLine(pen[2], arrowPoints[0], arrowPoints[1]);
				graphics.DrawLine(pen[3], arrowPoints[1], arrowPoints[2]);
				graphics.DrawLine(pen[3], arrowPoints[2], arrowPoints[3]);
				graphics.DrawLine(pen[3], arrowPoints[3], arrowPoints[4]);
				graphics.DrawLine(pen[0], arrowPoints[4], arrowPoints[5]);
				graphics.DrawLine(pen[3], arrowPoints[5], arrowPoints[6]);
				graphics.DrawLine(pen[0], arrowPoints[6], arrowPoints[0]);
				break;
			case Direction.Right:
				graphics.DrawLine(pen[1], arrowPoints[0], arrowPoints[1]);
				graphics.DrawLine(pen[0], arrowPoints[1], arrowPoints[2]);
				graphics.DrawLine(pen[0], arrowPoints[2], arrowPoints[3]);
				graphics.DrawLine(pen[0], arrowPoints[3], arrowPoints[4]);
				graphics.DrawLine(pen[3], arrowPoints[4], arrowPoints[5]);
				graphics.DrawLine(pen[0], arrowPoints[5], arrowPoints[6]);
				graphics.DrawLine(pen[3], arrowPoints[6], arrowPoints[0]);
				break;
			}
		}
	}
}
